@@ -20,6 +20,7 @@ impl Runner {
 
     pub(crate) fn run(&self, mode: &Mode, bins: &crate::BinRegistry) {
         let palette = crate::Palette::current();
+
         if self.cases.is_empty() {
             eprintln!(
                 "{}",
@@ -29,41 +30,63 @@ impl Runner {
             let failures: Vec<_> = self
                 .cases
                 .par_iter()
-                .filter_map(|c| match c.run(mode, bins) {
-                    Ok(status) => {
-                        let stderr = std::io::stderr();
-                        let mut stderr = stderr.lock();
-                        let _ = writeln!(
-                            stderr,
-                            "{} {} ... {}",
-                            palette.hint.paint("Testing"),
-                            c.path.display(),
-                            palette.info.paint("ok")
-                        );
-                        if !status.is_ok() {
-                            // Assuming `status` will print the newline
-                            let _ = write!(stderr, "{}", &status);
-                        }
-                        None
-                    }
-                    Err(status) => {
-                        let stderr = std::io::stderr();
-                        let mut stderr = stderr.lock();
-                        let _ = writeln!(
-                            stderr,
-                            "{} {} ... {}",
-                            palette.hint.paint("Testing"),
-                            c.path.display(),
-                            palette.error.paint("failed")
-                        );
-                        // Assuming `status` will print the newline
-                        let _ = write!(stderr, "{}", &status);
-                        Some(status)
-                    }
+                .flat_map(|c| {
+                    let results = c.run(mode, bins);
+
+                    let stderr = std::io::stderr();
+                    let mut stderr = stderr.lock();
+
+                    results
+                        .into_iter()
+                        .filter_map(|s| {
+                            match s {
+                                Ok(status) => {
+                                    let _ = writeln!(
+                                        stderr,
+                                        "{} {} ... {}",
+                                        palette.hint.paint("Testing"),
+                                        status.name(),
+                                        status.spawn.status.summary()
+                                    );
+                                    if !status.is_ok() {
+                                        // Assuming `status` will print the newline
+                                        let _ = write!(stderr, "{}", &status);
+                                    }
+                                    None
+                                }
+                                Err(status) => {
+                                    let _ = writeln!(
+                                        stderr,
+                                        "{} {} ... {}",
+                                        palette.hint.paint("Testing"),
+                                        status.name(),
+                                        status.spawn.status.summary()
+                                    );
+                                    // Assuming `status` will print the newline
+                                    let _ = write!(stderr, "{}", &status);
+                                    Some(status)
+                                }
+                            }
+                        })
+                        .collect::<Vec<_>>()
                 })
                 .collect();
 
             if !failures.is_empty() {
+                let stderr = std::io::stderr();
+                let mut stderr = stderr.lock();
+                let _ = writeln!(
+                    stderr,
+                    "{}",
+                    palette
+                        .hint
+                        .paint("Update snapshots with `TRYCMD=overwrite`"),
+                );
+                let _ = writeln!(
+                    stderr,
+                    "{}",
+                    palette.hint.paint("Debug output with `TRYCMD=dump`"),
+                );
                 panic!("{} of {} tests failed", failures.len(), self.cases.len());
             }
         }
@@ -78,20 +101,17 @@ impl Default for Runner {
 
 #[derive(Debug)]
 pub(crate) struct Case {
-    pub(crate) name: String,
     pub(crate) path: std::path::PathBuf,
-    pub(crate) expected: Option<crate::CommandStatus>,
+    pub(crate) expected: Option<crate::schema::CommandStatus>,
     pub(crate) timeout: Option<std::time::Duration>,
-    pub(crate) default_bin: Option<crate::Bin>,
-    pub(crate) env: crate::Env,
+    pub(crate) default_bin: Option<crate::schema::Bin>,
+    pub(crate) env: crate::schema::Env,
     pub(crate) error: Option<SpawnStatus>,
 }
 
 impl Case {
     pub(crate) fn with_error(path: std::path::PathBuf, error: impl std::fmt::Display) -> Self {
-        let name = path.display().to_string();
         Self {
-            name,
             path,
             expected: None,
             timeout: None,
@@ -101,71 +121,154 @@ impl Case {
         }
     }
 
-    pub(crate) fn run(&self, mode: &Mode, bins: &crate::BinRegistry) -> Result<Output, Output> {
-        let mut output = Output::default();
-
-        if self.expected == Some(crate::CommandStatus::Skip) {
+    pub(crate) fn run(
+        &self,
+        mode: &Mode,
+        bins: &crate::BinRegistry,
+    ) -> Vec<Result<Output, Output>> {
+        if self.expected == Some(crate::schema::CommandStatus::Skipped) {
+            let output = Output::sequence(self.path.clone());
             assert_eq!(output.spawn.status, SpawnStatus::Skipped);
-            return Ok(output);
-        }
-        if let Some(err) = self.error.clone() {
-            output.spawn.status = err;
-            return Err(output);
+            return vec![Ok(output)];
         }
 
-        let mut run = crate::TryCmd::load(&self.path).map_err(|e| output.clone().error(e))?;
-        if run.bin.is_none() {
-            run.bin = self.default_bin.clone()
+        if let Some(err) = self.error.clone() {
+            let mut output = Output::step(self.path.clone(), "setup".into());
+            output.spawn.status = err;
+            return vec![Err(output)];
         }
-        run.bin = run
-            .bin
+
+        let mut sequence = match crate::schema::TryCmd::load(&self.path) {
+            Ok(sequence) => sequence,
+            Err(e) => {
+                let output = Output::step(self.path.clone(), "setup".into());
+                return vec![Err(output.error(e))];
+            }
+        };
+
+        if sequence.steps.is_empty() {
+            let output = Output::sequence(self.path.clone());
+            assert_eq!(output.spawn.status, SpawnStatus::Skipped);
+            return vec![Ok(output)];
+        }
+
+        let fs_context = match crate::FilesystemContext::new(
+            &self.path,
+            sequence.fs.base.as_deref(),
+            sequence.fs.sandbox(),
+            mode,
+        ) {
+            Ok(fs_context) => fs_context,
+            Err(e) => {
+                let output = Output::step(self.path.clone(), "setup".into());
+                return vec![Err(
+                    output.error(format!("Failed to initialize sandbox: {}", e))
+                )];
+            }
+        };
+        let cwd = match fs_context
+            .path()
+            .map(|p| sequence.fs.rel_cwd().map(|rel| p.join(rel)))
+            .transpose()
+        {
+            Ok(cwd) => cwd,
+            Err(e) => {
+                let output = Output::step(self.path.clone(), "setup".into());
+                return vec![Err(output.error(e))];
+            }
+        };
+
+        let mut outputs = Vec::with_capacity(sequence.steps.len());
+        let mut prior_step_failed = false;
+        for step in &mut sequence.steps {
+            if prior_step_failed {
+                step.expected_status = Some(crate::schema::CommandStatus::Skipped);
+            }
+
+            let step_status = self.run_step(step, cwd.as_deref(), mode, bins);
+            if step_status.is_err() {
+                prior_step_failed = true;
+            }
+            outputs.push(step_status);
+        }
+
+        if sequence.fs.sandbox() {
+            let mut ok = true;
+            let mut output = Output::step(self.path.clone(), "teardown".into());
+
+            output.fs = match self.validate_fs(
+                fs_context.path().expect("sandbox must be filled"),
+                output.fs,
+                mode,
+            ) {
+                Ok(fs) => fs,
+                Err(fs) => {
+                    ok = false;
+                    fs
+                }
+            };
+            if let Err(err) = fs_context.close() {
+                ok = false;
+                output.fs.context.push(FileStatus::Failure(format!(
+                    "Failed to cleanup sandbox: {}",
+                    err
+                )));
+            }
+
+            let output = if ok {
+                output.spawn.status = SpawnStatus::Ok;
+                Ok(output)
+            } else {
+                output.spawn.status = SpawnStatus::Failure("Files left in unexpected state".into());
+                Err(output)
+            };
+            outputs.push(output);
+        }
+
+        outputs
+    }
+
+    pub(crate) fn run_step(
+        &self,
+        step: &mut crate::schema::Step,
+        cwd: Option<&std::path::Path>,
+        mode: &Mode,
+        bins: &crate::BinRegistry,
+    ) -> Result<Output, Output> {
+        let output = if let Some(id) = step.id.clone() {
+            Output::step(self.path.clone(), id)
+        } else {
+            Output::sequence(self.path.clone())
+        };
+
+        let mut bin = step.bin.take();
+        if bin.is_none() {
+            bin = self.default_bin.clone()
+        }
+        bin = bin
             .map(|name| bins.resolve_bin(name))
             .transpose()
             .map_err(|e| output.clone().error(e))?;
-        if run.timeout.is_none() {
-            run.timeout = self.timeout;
+        step.bin = bin;
+        if step.timeout.is_none() {
+            step.timeout = self.timeout;
         }
         if self.expected.is_some() {
-            run.status = self.expected;
+            step.expected_status = self.expected;
         }
-        run.env.update(&self.env);
+        step.env.update(&self.env);
 
-        let stdin_path = self.path.with_extension("stdin");
-        let stdin = if stdin_path.exists() {
-            Some(
-                File::read_from(&stdin_path, run.binary)
-                    .map_err(|e| {
-                        output.clone().error(format!(
-                            "Failed to read {}: {}",
-                            stdin_path.display(),
-                            e
-                        ))
-                    })?
-                    .into_bytes(),
-            )
-        } else {
-            None
-        };
+        if step.expected_status() == crate::schema::CommandStatus::Skipped {
+            assert_eq!(output.spawn.status, SpawnStatus::Skipped);
+            return Ok(output);
+        }
 
-        let fs = crate::FilesystemContext::new(
-            &self.path,
-            run.fs.base.as_deref(),
-            run.fs.sandbox(),
-            mode,
-        )
-        .map_err(|e| {
-            output
-                .clone()
-                .error(format!("Failed to initialize sandbox: {}", e))
-        })?;
-        let cmd_output = run
-            .to_output(stdin, fs.path())
-            .map_err(|e| output.clone().error(e))?;
+        let cmd_output = step.to_output(cwd).map_err(|e| output.clone().error(e))?;
         let output = output.output(cmd_output);
 
         // For dump mode's sake, allow running all
         let mut ok = output.is_ok();
-        let mut output = match self.validate_spawn(output, run.status()) {
+        let mut output = match self.validate_spawn(output, step.expected_status()) {
             Ok(output) => output,
             Err(output) => {
                 ok = false;
@@ -173,11 +276,11 @@ impl Case {
             }
         };
         if let Some(mut stdout) = output.stdout {
-            if !run.binary {
+            if !step.binary {
                 stdout = stdout.utf8();
             }
             if stdout.is_ok() {
-                stdout = match self.validate_stream(stdout, mode) {
+                stdout = match self.validate_stream(stdout, step.expected_stdout.as_ref(), mode) {
                     Ok(stdout) => stdout,
                     Err(stdout) => {
                         ok = false;
@@ -188,11 +291,11 @@ impl Case {
             output.stdout = Some(stdout);
         }
         if let Some(mut stderr) = output.stderr {
-            if !run.binary {
+            if !step.binary {
                 stderr = stderr.utf8();
             }
             if stderr.is_ok() {
-                stderr = match self.validate_stream(stderr, mode) {
+                stderr = match self.validate_stream(stderr, step.expected_stderr.as_ref(), mode) {
                     Ok(stderr) => stderr,
                     Err(stderr) => {
                         ok = false;
@@ -202,23 +305,6 @@ impl Case {
             }
             output.stderr = Some(stderr);
         }
-        if run.fs.sandbox() {
-            output.fs =
-                match self.validate_fs(fs.path().expect("sandbox must be filled"), output.fs, mode)
-                {
-                    Ok(fs) => fs,
-                    Err(fs) => {
-                        ok = false;
-                        fs
-                    }
-                };
-        }
-        if let Err(err) = fs.close() {
-            output.fs.context.push(FileStatus::Failure(format!(
-                "Failed to cleanup sandbox: {}",
-                err
-            )));
-        }
 
         if ok {
             Ok(output)
@@ -230,27 +316,27 @@ impl Case {
     fn validate_spawn(
         &self,
         mut output: Output,
-        expected: crate::CommandStatus,
+        expected: crate::schema::CommandStatus,
     ) -> Result<Output, Output> {
         let status = output.spawn.exit.expect("bale out before now");
         match expected {
-            crate::CommandStatus::Pass => {
+            crate::schema::CommandStatus::Success => {
                 if !status.success() {
                     output.spawn.status = SpawnStatus::Expected("success".into());
                 }
             }
-            crate::CommandStatus::Fail => {
+            crate::schema::CommandStatus::Failed => {
                 if status.success() || status.code().is_none() {
                     output.spawn.status = SpawnStatus::Expected("failure".into());
                 }
             }
-            crate::CommandStatus::Interrupted => {
+            crate::schema::CommandStatus::Interrupted => {
                 if status.code().is_some() {
                     output.spawn.status = SpawnStatus::Expected("interrupted".into());
                 }
             }
-            crate::CommandStatus::Skip => unreachable!("handled earlier"),
-            crate::CommandStatus::Code(expected_code) => {
+            crate::schema::CommandStatus::Skipped => unreachable!("handled earlier"),
+            crate::schema::CommandStatus::Code(expected_code) => {
                 if Some(expected_code) != status.code() {
                     output.spawn.status = SpawnStatus::Expected(expected_code.to_string());
                 }
@@ -260,7 +346,12 @@ impl Case {
         Ok(output)
     }
 
-    fn validate_stream(&self, mut stream: Stream, mode: &Mode) -> Result<Stream, Stream> {
+    fn validate_stream(
+        &self,
+        mut stream: Stream,
+        expected_content: Option<&crate::File>,
+        mode: &Mode,
+    ) -> Result<Stream, Stream> {
         if let Mode::Dump(root) = mode {
             let stdout_path = root.join(
                 self.path
@@ -270,51 +361,36 @@ impl Case {
             );
             stream.content.write_to(&stdout_path).map_err(|e| {
                 let mut stream = stream.clone();
-                stream.status = StreamStatus::Failure(format!(
-                    "Failed to read {}: {}",
-                    stdout_path.display(),
-                    e
-                ));
+                stream.status = StreamStatus::Failure(e);
                 stream
             })?;
-        } else {
-            let stdout_path = self.path.with_extension(stream.stream.as_str());
-            if stdout_path.exists() {
-                let expected_content = File::read_from(&stdout_path, stream.content.is_binary())
-                    .map_err(|e| {
-                        let mut stream = stream.clone();
-                        stream.status = StreamStatus::Failure(format!(
-                            "Failed to read {}: {}",
-                            stdout_path.display(),
-                            e
-                        ));
-                        stream
-                    })?;
-
-                if let File::Text(e) = &expected_content {
-                    stream.content = stream.content.map_text(|t| crate::elide::normalize(t, e));
-                }
-                if stream.content != expected_content {
-                    match mode {
-                        Mode::Fail => {
-                            stream.status = StreamStatus::Expected(expected_content);
-                            return Err(stream);
-                        }
-                        Mode::Overwrite => {
+        } else if let Some(expected_content) = expected_content {
+            if let crate::File::Text(e) = &expected_content {
+                stream.content = stream.content.map_text(|t| crate::elide::normalize(t, e));
+            }
+            if stream.content != *expected_content {
+                match mode {
+                    Mode::Fail => {
+                        stream.status = StreamStatus::Expected(expected_content.clone());
+                        return Err(stream);
+                    }
+                    Mode::Overwrite => {
+                        let stdout_path = self.path.with_extension(stream.stream.as_str());
+                        if stdout_path.exists() {
                             stream.content.write_to(&stdout_path).map_err(|e| {
                                 let mut stream = stream.clone();
-                                stream.status = StreamStatus::Failure(format!(
-                                    "Failed to write {}: {}",
-                                    stdout_path.display(),
-                                    e
-                                ));
+                                stream.status = StreamStatus::Failure(e);
                                 stream
                             })?;
-                            stream.status = StreamStatus::Expected(expected_content);
+                            stream.status = StreamStatus::Expected(expected_content.clone());
                             return Ok(stream);
+                        } else {
+                            // `.trycmd` files do not support overwrite, see issue #23
+                            stream.status = StreamStatus::Expected(expected_content.clone());
+                            return Err(stream);
                         }
-                        Mode::Dump(_) => unreachable!("handled earlier"),
                     }
+                    Mode::Dump(_) => unreachable!("handled earlier"),
                 }
             }
         }
@@ -459,26 +535,14 @@ impl Case {
                 }
             }
             FileType::File => {
-                let expected_content = File::read_from(&expected_path, true)
-                    .map_err(|e| {
-                        FileStatus::Failure(format!(
-                            "Failed to read {}: {}",
-                            expected_path.display(),
-                            e
-                        ))
-                    })?
+                let expected_content = crate::File::read_from(&expected_path, true)
+                    .map_err(FileStatus::Failure)?
                     .try_utf8();
-                let mut actual_content = File::read_from(&actual_path, true)
-                    .map_err(|e| {
-                        FileStatus::Failure(format!(
-                            "Failed to read {}: {}",
-                            actual_path.display(),
-                            e
-                        ))
-                    })?
+                let mut actual_content = crate::File::read_from(&actual_path, true)
+                    .map_err(FileStatus::Failure)?
                     .try_utf8();
 
-                if let File::Text(e) = &expected_content {
+                if let crate::File::Text(e) = &expected_content {
                     actual_content = actual_content.map_text(|t| crate::elide::normalize(t, e));
                 }
                 if expected_content != actual_content {
@@ -502,6 +566,8 @@ impl Case {
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub(crate) struct Output {
+    path: std::path::PathBuf,
+    id: Option<String>,
     spawn: Spawn,
     stdout: Option<Stream>,
     stderr: Option<Stream>,
@@ -509,18 +575,43 @@ pub(crate) struct Output {
 }
 
 impl Output {
+    fn sequence(path: std::path::PathBuf) -> Self {
+        Self {
+            path,
+            id: None,
+            spawn: Spawn {
+                exit: None,
+                status: SpawnStatus::Skipped,
+            },
+            stdout: None,
+            stderr: None,
+            fs: Default::default(),
+        }
+    }
+
+    fn step(path: std::path::PathBuf, step: String) -> Self {
+        Self {
+            path,
+            id: Some(step),
+            spawn: Default::default(),
+            stdout: None,
+            stderr: None,
+            fs: Default::default(),
+        }
+    }
+
     fn output(mut self, output: std::process::Output) -> Self {
         self.spawn.exit = Some(output.status);
         assert_eq!(self.spawn.status, SpawnStatus::Skipped);
         self.spawn.status = SpawnStatus::Ok;
         self.stdout = Some(Stream {
             stream: Stdio::Stdout,
-            content: File::Binary(output.stdout),
+            content: crate::File::Binary(output.stdout),
             status: StreamStatus::Ok,
         });
         self.stderr = Some(Stream {
             stream: Stdio::Stderr,
-            content: File::Binary(output.stderr),
+            content: crate::File::Binary(output.stderr),
             status: StreamStatus::Ok,
         });
         self
@@ -533,23 +624,16 @@ impl Output {
 
     fn is_ok(&self) -> bool {
         self.spawn.is_ok()
-            && self.stdout.as_ref().map(|s| s.is_ok()).unwrap_or_default()
-            && self.stderr.as_ref().map(|s| s.is_ok()).unwrap_or_default()
+            && self.stdout.as_ref().map(|s| s.is_ok()).unwrap_or(true)
+            && self.stderr.as_ref().map(|s| s.is_ok()).unwrap_or(true)
             && self.fs.is_ok()
     }
-}
 
-impl Default for Output {
-    fn default() -> Self {
-        Self {
-            spawn: Spawn {
-                exit: None,
-                status: SpawnStatus::Skipped,
-            },
-            stdout: None,
-            stderr: None,
-            fs: Default::default(),
-        }
+    fn name(&self) -> String {
+        self.id
+            .as_deref()
+            .map(|id| format!("{}:{}", self.path.display(), id))
+            .unwrap_or_else(|| self.path.display().to_string())
     }
 }
 
@@ -580,6 +664,15 @@ impl Spawn {
     }
 }
 
+impl Default for Spawn {
+    fn default() -> Self {
+        Self {
+            exit: None,
+            status: SpawnStatus::Skipped,
+        }
+    }
+}
+
 impl std::fmt::Display for Spawn {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let palette = crate::Palette::current();
@@ -649,12 +742,21 @@ impl SpawnStatus {
             Self::Failure(_) | Self::Expected(_) => false,
         }
     }
+
+    fn summary(&self) -> impl std::fmt::Display {
+        let palette = crate::Palette::current();
+        match self {
+            Self::Ok => palette.info.paint("ok"),
+            Self::Skipped => palette.warn.paint("ignored"),
+            Self::Failure(_) | Self::Expected(_) => palette.error.paint("failed"),
+        }
+    }
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 struct Stream {
     stream: Stdio,
-    content: File,
+    content: crate::File,
     status: StreamStatus,
 }
 
@@ -693,7 +795,9 @@ impl std::fmt::Display for Stream {
                 #[allow(unused_mut)]
                 let mut rendered = false;
                 #[cfg(feature = "diff")]
-                if let (File::Text(expected), File::Text(actual)) = (&expected, &self.content) {
+                if let (crate::File::Text(expected), crate::File::Text(actual)) =
+                    (&expected, &self.content)
+                {
                     let diff =
                         crate::diff::diff(expected, actual, self.stream, self.stream, palette);
                     writeln!(f, "{}", diff)?;
@@ -717,7 +821,7 @@ impl std::fmt::Display for Stream {
 enum StreamStatus {
     Ok,
     Failure(String),
-    Expected(File),
+    Expected(crate::File),
 }
 
 impl StreamStatus {
@@ -797,8 +901,8 @@ enum FileStatus {
     ContentMismatch {
         expected_path: std::path::PathBuf,
         actual_path: std::path::PathBuf,
-        expected_content: File,
-        actual_content: File,
+        expected_content: crate::File,
+        actual_content: crate::File,
     },
 }
 
@@ -870,7 +974,7 @@ impl std::fmt::Display for FileStatus {
                 #[allow(unused_mut)]
                 let mut rendered = false;
                 #[cfg(feature = "diff")]
-                if let (File::Text(expected), File::Text(actual)) =
+                if let (crate::File::Text(expected), crate::File::Text(actual)) =
                     (&expected_content, &actual_content)
                 {
                     let diff = crate::diff::diff(
@@ -956,92 +1060,3 @@ impl Mode {
         Ok(())
     }
 }
-
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub(crate) enum File {
-    Binary(Vec<u8>),
-    Text(String),
-}
-
-impl File {
-    pub(crate) fn read_from(path: &std::path::Path, binary: bool) -> Result<Self, std::io::Error> {
-        let data = if binary {
-            let data = std::fs::read(&path)?;
-            Self::Binary(data)
-        } else {
-            let data = std::fs::read_to_string(&path)?;
-            let data = normalize_line_endings::normalized(data.chars()).collect();
-            Self::Text(data)
-        };
-        Ok(data)
-    }
-
-    pub(crate) fn write_to(&self, path: &std::path::Path) -> Result<(), std::io::Error> {
-        std::fs::write(path, self.as_bytes())
-    }
-
-    pub(crate) fn map_text(self, op: impl FnOnce(&str) -> String) -> Self {
-        match self {
-            Self::Binary(data) => Self::Binary(data),
-            Self::Text(data) => Self::Text(op(&data)),
-        }
-    }
-
-    pub(crate) fn is_binary(&self) -> bool {
-        match self {
-            Self::Binary(_) => true,
-            Self::Text(_) => false,
-        }
-    }
-
-    pub(crate) fn utf8(&mut self) -> Result<(), std::str::Utf8Error> {
-        match self {
-            Self::Binary(data) => {
-                let data = String::from_utf8(data.clone()).map_err(|e| e.utf8_error())?;
-                let data = normalize_line_endings::normalized(data.chars()).collect();
-                *self = Self::Text(data);
-                Ok(())
-            }
-            Self::Text(_) => Ok(()),
-        }
-    }
-
-    pub(crate) fn try_utf8(self) -> Self {
-        match self {
-            Self::Binary(data) => match String::from_utf8(data) {
-                Ok(data) => {
-                    let data = normalize_line_endings::normalized(data.chars()).collect();
-                    Self::Text(data)
-                }
-                Err(err) => {
-                    let data = err.into_bytes();
-                    Self::Binary(data)
-                }
-            },
-            Self::Text(data) => Self::Text(data),
-        }
-    }
-
-    pub(crate) fn as_bytes(&self) -> &[u8] {
-        match self {
-            Self::Binary(data) => data,
-            Self::Text(data) => data.as_bytes(),
-        }
-    }
-
-    pub(crate) fn into_bytes(self) -> Vec<u8> {
-        match self {
-            Self::Binary(data) => data,
-            Self::Text(data) => data.into_bytes(),
-        }
-    }
-}
-
-impl std::fmt::Display for File {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        match self {
-            Self::Binary(data) => String::from_utf8_lossy(data).fmt(f),
-            Self::Text(data) => data.fmt(f),
-        }
-    }
-}
